/**********************************************************************
  GamessLog - parses GAMESS-US log files

  Copyright (C) 2009 Marcus D. Hanwell

  This file is part of the Avogadro molecular editor project.
  For more information, see <http://avogadro.sourceforge.net/>

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  02110-1301, USA.
 **********************************************************************/

#ifndef GAMESSLOG_H
#define GAMESSLOG_H

#include <Eigen/Core>
#include <vector>

class QString;
class QTextStream;

namespace Avogadro
{
  class BasisSet;

  class GamessLog
  {
  public:
    GamessLog(QString filename, BasisSet *basis);
    ~GamessLog();
    void outputAll();
  private:
    QTextStream *m_in;
    void processLine();
    void load(BasisSet* basis);

    void readAtoms();   // Read in the atoms and their positions
    void readBasis();   // Read the basis set in
    void readMOs();     // Read in MOs

    int primitiveType(const QString &string);  // Return the primitive type
    std::vector<int> readArrayI(unsigned int n);
    std::vector<double> readArrayD(unsigned int n);

    int m_numElectrons;      // Number of electrons
    int m_numBasisFunctions; // Number of cartesian basis functions
    int m_charge;            // Charge of the molecule
    std::vector<QString> m_atomSymbol;
    std::vector<int> m_atomNum;
    std::vector<Eigen::Vector3d> m_atomPos;
    std::vector<int> m_primitiveTypes;
    std::vector<int> m_shellNums;
    std::vector<int> m_shellToAtom;
    std::vector<double> m_a;
    std::vector<double> m_c;
    std::vector<double> m_csp;
    std::vector<double> m_orbitalEnergy;
    Eigen::MatrixXd m_eigenVectors;
    std::vector<double> m_MOcoeffs;
  };

} // End namespace Avogadro

#endif
